

class CouldNotFindTorPortException(Exception):
    def __init__(self, message: str = None):
        if message is None:
            message = "Could not connect to Tor using the expected ports. Is Tor installed on this machine?"
        super().__init__(message)






class InvalidHTTPMethodException(Exception):
    def __init__(self, message: str = None, invalid_method: str = None):
        if message is None:
            message = "An invalid HTTP method was provided to the OnionQueue"
            if isinstance(invalid_method, str):
                message += ": {}. ".format(invalid_method)
            else:
                message += ". "
            message += "Valid HTTP methods include: GET, HEAD, POST, PUT, DELETE, OPTIONS, or PATCH."
        super().__init__(message)





import random
import time

import requests
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry

from onionrequests.TorPort import TorPort


class OnionSession(requests.Session):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # Generate a unique username and password to connect to Tor.
        random.seed(time.time_ns())
        username = str(time.time_ns()) + "".join(random.choice("ABCDEFGHIJKLMNOPQRSTUVWXYZ") for _ in range(10))
        password = str(time.time_ns()) + "".join(random.choice("ABCDEFGHIJKLMNOPQRSTUVWXYZ") for _ in range(10))

        # Figure out which port number Tor's SOCKS proxy is using, if available.
        port = TorPort.get_tor_port()

        # Tell the session to use HTTP through a Tor proxy.
        retry = Retry(connect=10, backoff_factor=1.0)
        adapter = HTTPAdapter(max_retries=retry)
        self.mount('http://', adapter)
        self.mount('https://', adapter)
        self.proxies = {
            'http': 'socks5://{}:{}@localhost:{}'.format(username, password, port),
            'https': 'socks5://{}:{}@localhost:{}'.format(username, password, port)
        }



import socket

from onionrequests.exceptions import CouldNotFindTorPortException


class TorPort:
    __CONST_TOR_PORT = None

    # If your Tor port isn't in this list, you can append it.
    # The first valid port found from this list is assumed to be the Tor SOCKS port.
    __potential_tor_ports = [
        9050,  # Default for Linux.
        9150,  # Default for Windows.
    ]

    @classmethod
    def get_tor_port(cls) -> int:
        if isinstance(cls.__CONST_TOR_PORT, int):
            return cls.__CONST_TOR_PORT

        for port_num in cls.__potential_tor_ports:
            try:
                s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                s.connect(("127.0.0.1", port_num))
                cls.__CONST_TOR_PORT = port_num
                return port_num
            except:
                continue

        raise CouldNotFindTorPortException()

    @classmethod
    def add_potential_tor_port(cls, port_num: int):
        assert isinstance(port_num, int)
        cls.__potential_tor_ports.insert(0, port_num)
